using System;
using UnityEngine;
using UnityEngine.Events;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
namespace Mod
{
    public class Mod
    {
        public static void Main()
        {
            //This method is the entry point. 
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Pistol"), //item to derive from
                    NameOverride = "Penny Pistol", //new item name with a suffix to assure it is globally unique
                    DescriptionOverride = "Similar to a pistol, but does little damage and a lot of knockback.", //new item description
                    CategoryOverride = ModAPI.FindCategory("Firearms"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("ShovetolThumb.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        //setting the sprite
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Shovetol.png");

                        //getting the FirearmBehaviour for later manipulation
                        var firearm = Instance.GetComponent<FirearmBehaviour>();

                        //creating a custom cartridge for the gun
                        Cartridge customCartridge = ModAPI.FindCartridge("9mm"); //load a copy of the 9mm cartridge
                        customCartridge.name = "10mm Knock"; //set a name
                        customCartridge.Damage *= 0.002f; //change the damage however you like
                        customCartridge.StartSpeed *= 0.05f; //change the bullet velocity
                        customCartridge.PenetrationRandomAngleMultiplier *= 0.5f; //change the accuracy when the bullet travels through an object
                        customCartridge.Recoil *= 2f; //change the recoil
                        customCartridge.ImpactForce *= 500f; //change how much the bullet pushes the target

                        //set the cartridge to the FirearmBehaviour
                        firearm.Cartridge = customCartridge;

                        //set the new gun sounds. this is an array of AudioClips that is picked from at random when shot
                        firearm.ShotSounds = new AudioClip[]
                        {
                            ModAPI.LoadSound("stolShot.wav"),
                        };

                        // set the collision box to the new sprite shape
                        // this is the easiest way to fix your collision shape, but it also the slowest.
                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Human"), //item to derive from
                    NameOverride = "Human (CTOD)", //new item name with a suffix to assure it is globally unique
                    DescriptionOverride = "Like the normal human, but with better damage physics.", //new item description
                    CategoryOverride = ModAPI.FindCategory("Entities"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("humanthumb.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        //load textures for each layer (see Human textures folder in this repository)
                        string[] conditions = { "badback", "old", "strong", "braindamage", "numbed", "resilient" };
                        string cond = "";
                        if (UnityEngine.Random.Range(0f, 1f) > 0.5f)
                            cond = conditions[UnityEngine.Random.Range(0, conditions.Length)];
                        var skin = ModAPI.LoadTexture("hm1" + cond + ".png");
                        var flesh = ModAPI.LoadTexture("hm2.png");
                        var bone = ModAPI.LoadTexture("hm3.png");

                        //get person
                        var person = Instance.GetComponent<PersonBehaviour>();
                        modperson modp = Instance.AddComponent<modperson>();

                        //use the helper function to set each texture
                        //parameters are as follows: 
                        //  skin texture, flesh texture, bone texture, sprite scale
                        //you can pass "null" to fall back to the original texture
                        person.SetBodyTextures(skin, flesh, bone, 1);

                        //change procedural damage colours if they interfere with your texture (rgb 0-255)
                        person.SetBruiseColor(157, 3, 27); //main bruise colour. purple-ish by default
                        person.SetSecondBruiseColor(154, 0, 7); //second bruise colour. red by default
                        person.SetThirdBruiseColor(143, 97, 134); // third bruise colour. light yellow by default
                        person.SetRottenColour(202, 199, 104); // rotten/zombie colour. light yellow/green by default
                        person.SetBloodColour(108, 0, 4); // blood colour. dark red by default. note that this does not change decal nor particle effect colours. it only affects the procedural blood color which may or may not be rendered
                        foreach (var p in person.Limbs)
                        {
                            List<ContextMenuButton> buttonsList = p.PhysicalBehaviour.ContextMenuOptions.Buttons;
                            var heartAttack = new ContextMenuButton(() => modp.heartHealth == 1f, "heartAttack", "Heart Attack", "Give the selected human a heart attack", new UnityAction[1]
                            {
                            (UnityAction) (() => modp.heartHealth = 0f)
                            });
                            heartAttack.LabelWhenMultipleAreSelected = "Give Heart Attack";
                            ///////////////////////////////////////////////////////////////////////////////
                            var healHeart = new ContextMenuButton(() => modp.heartHealth != 1f, "healHeart", "Heal Heart", "Heal the humans heart", new UnityAction[1]
                            {
                            (UnityAction) (() => modp.heartHealth = 1f)
                            });
                            healHeart.LabelWhenMultipleAreSelected = "Heal Heart";
                            ////////////////////////////////////////////////////////////////////////
                            var removeBdmg = new ContextMenuButton(() => modp.brainDmgMultiplier != 0f, "removeBdmg", "Remove Brain Damage", "Reset the brain damage of the human", new UnityAction[1]
                            {
                            (UnityAction) (() => modp.brainDmgMultiplier = 0f)
                            });
                            heartAttack.LabelWhenMultipleAreSelected = "Remove Brain Damage";
                            ///////////////////////////////////////////////////////////////////////
                            var seizure = new ContextMenuButton(() => modp.winceAmount <= 0f, "seizure", "Give Seizure", "Give the human a seizure", new UnityAction[1]
                            {
                            (UnityAction) (() => modp.winceAmount = 6f)
                            });
                            seizure.LabelWhenMultipleAreSelected = "Give Seizures";
                            /////////////////////////////////////////////////////////////////////
                            var asphyxiate = new ContextMenuButton(() => modp.lungHealth == 1f, "asphyxiate", "Asphyxiate", "Remove the lungs of the human", new UnityAction[1]
                            {
                            (UnityAction) (() => {
                                modp.lungHealth = 0f;
                                })
                            });
                            asphyxiate.LabelWhenMultipleAreSelected = "Asphyxiate";
                            //////////////////////////////////////////////////////////
                            var healLungs = new ContextMenuButton(() => modp.lungHealth != 1f, "healLungs", "Heal Lungs", "Heal the lungs of the human", new UnityAction[1]
                            {
                            (UnityAction) (() => {
                                modp.lungHealth = 1f;
                                modp.lungHealthLossRate = 0f;
                                })
                            });
                            healLungs.LabelWhenMultipleAreSelected = "Heal Lungs";
                            //////////////////////////////////////////////////////////
                            var killLiver = new ContextMenuButton(() => modp.liverHealth == 1f, "killLiver", "Kill liver", "Kill the humans liver", new UnityAction[1]
                            {
                            (UnityAction) (() => modp.liverHealth = 0f)
                            });
                            seizure.LabelWhenMultipleAreSelected = "Kill livers";
                            //////////////////////////////////////////
                            var healLiver = new ContextMenuButton(() => modp.liverHealth != 1f, "healLiver", "Heal liver", "Heal the humans liver", new UnityAction[1]
                            {
                            (UnityAction) (() => modp.liverHealth = 1f)
                            });
                            seizure.LabelWhenMultipleAreSelected = "Heal livers";
                            buttonsList.Add(heartAttack);
                            buttonsList.Add(healHeart);
                            buttonsList.Add(removeBdmg);
                            buttonsList.Add(seizure);
                            buttonsList.Add(asphyxiate);
                            buttonsList.Add(healLungs);
                            buttonsList.Add(killLiver);
                            buttonsList.Add(healLiver);
                        }
                        
                        //change damage physics
                        foreach (LimbBehaviour limb in person.Limbs)
                        {
                            string a = limb.gameObject.name;
                            if (a != "Head" && a != "UpperBody" && a != "MiddleBody" && a != "LowerBody")
                            {
                                //limb.BreakingThreshold *= 0.33f;
                            }
                            else
                            {
                                if(cond == "badback")
                                {
                                    limb.BreakingThreshold *= 0.5f;
                                    limb.BaseStrength *= 0.5f;
                                }
                            }
                            if(cond == "old")
                            {
                                limb.BreakingThreshold *= 0.7f;
                                limb.BaseStrength *= 0.55f;
                                limb.FakeUprightForce *= 0.5f;
                            }
                            else if (cond == "strong")
                            {
                                limb.BreakingThreshold *= 1.1f;
                                limb.BaseStrength *= 2f;
                                limb.FakeUprightForce *= 1.3f;
                            }
                            else if (cond == "braindamage")
                            {
                                modp.brainDmgMultiplier = 0.5f;
                            }
                            else if (cond == "numbed")
                            {
                                if(UnityEngine.Random.Range(0f, 1f) > 0.66f)
                                {
                                    limb.BaseStrength = 0f;
                                }
                            }
                            else if (cond == "resilient")
                            {
                                limb.BreakingThreshold *= 1.4f;
                                limb.Health *= 1.5f;
                                limb.InitialHealth *= 1.5f;
                                limb.CirculationBehaviour.BleedingRate *= 0.55f;
                                limb.BloodMuscleStrengthRatio *= 0.45f;
                            }
                            modlimb mod = limb.gameObject.AddComponent<modlimb>();
                            mod.PB = person;
                        }
                        switch(cond)
                        {
                            case "badback":
                                ModAPI.Notify("This person has a frail back.");
                                break;
                            case "old":
                                ModAPI.Notify("This person is old and weak.");
                                break;
                            case "strong":
                                ModAPI.Notify("This person is stronger than usual.");
                                break;
                            case "braindamage":
                                ModAPI.Notify("This person has a damaged brain and will have seizures.");
                                break;
                            case "numbed":
                                ModAPI.Notify("This person has suffered a stroke and cant move some of their limbs.");
                                break;
                            case "resilient":
                                ModAPI.Notify("This person can survive severe damage.");
                                break;
                        }
                    }
                }
            );
        }
    }
}